import binascii
import datetime
import json
from hashlib import sha256

from ecdsa.util import sigencode_der, sigdecode_der
from flask import Flask, render_template, request, make_response, redirect
import ecdsa
from werkzeug.exceptions import Forbidden

app = Flask(__name__)

with open('cookie_key.txt', 'r') as f:
    cookie_key = f.read().strip()

with open('admin_pwd.txt', 'r') as f:
    admin_pwd = f.read().strip()

sk = ecdsa.SigningKey.from_string(binascii.unhexlify(cookie_key), curve=ecdsa.SECP256k1)


def check_login(login, password):
    if login == 'user' and password == 'user':
        return 'user'
    elif login == 'admin' and password == admin_pwd:
        return 'admin'

    raise Forbidden('Invalid login or password')


def verify_auth_cookie():
    auth_cookie = request.cookies['auth']
    a1, a2 = auth_cookie.split('.', 1)

    if not sk.verifying_key.verify(binascii.unhexlify(a2), binascii.unhexlify(a1), hashfunc=sha256, sigdecode=sigdecode_der):
        raise Forbidden("Invalid auth cookie.")

    auth_user = json.loads(binascii.unhexlify(a1))['auth_login']
    return auth_user


@app.route("/panel")
def route_panel():
    if 'auth' not in request.cookies:
        return redirect("/")

    auth_user = verify_auth_cookie()
    flag = 'access denied for that account'

    if auth_user == 'admin':
        with open('flag.txt', 'r') as f:
            flag = f.read().strip()

    return render_template('panel.html', auth_user=auth_user, flag=flag)


@app.route("/logout")
def route_logout():
    resp = make_response(redirect("/panel"))
    resp.set_cookie('auth', '', expires=datetime.datetime.now() - datetime.timedelta(hours=24))

    return resp


@app.route("/check", methods=['POST'])
def route_check():
    login = request.form['login']
    password = request.form['password']

    auth_login = check_login(login, password)
    obj_b = json.dumps({"auth_login": auth_login, "sig_alg": "ECDSA/secp256k1/SHA256"}).encode('utf-8')
    obj_sig = sk.sign(obj_b, hashfunc=sha256, sigencode=sigencode_der)
    auth_cookie = obj_b.hex() + '.' + obj_sig.hex()

    resp = make_response(redirect("/panel"))
    resp.set_cookie('auth', auth_cookie)

    return resp


@app.route("/")
def route_login():
    try:
        verify_auth_cookie()
    except:
        pass
    else:
        return redirect('/panel')

    return render_template('login.html')


if __name__ == "__main__":
    app.run(host='0.0.0.0', port=8080)
